@@ -1,8 +1,10 @@
 import contextlib
 import datetime
-import time
 import functools
 import heapq
+import time
+from numbers import Number
+
 
 class Timeline(object):
     def __init__(self, start_time=None):
@@ -53,6 +55,8 @@ class Timeline(object):
         """
         Sleeps a given number of seconds in the virtual timeline
         """
+        if not isinstance(seconds, Number):
+            raise ValueError("Invalid number of seconds specified: {0!r}".format(seconds))
         if seconds < 0:
             raise ValueError("Cannot sleep negative number of seconds")
         if self._time_factor == 0:
@@ -68,6 +72,14 @@ class Timeline(object):
         while self._scheduled:
             self.sleep(max(0, self._scheduled[0][0]-self.time()))
 
+    def sleep_stop_first_scheduled(self, sleep_seconds):
+        """
+        Sleeps the given amount of time, but wakes up if a scheduled event exists before the destined end time
+        """
+        if self._scheduled:
+            sleep_seconds = min(max(0, self._scheduled[0][0]-self.time()), sleep_seconds)
+        self.sleep(sleep_seconds)
+
     def trigger_past_callbacks(self):
         current_time = self.time()
         while self._scheduled and self._scheduled[0][0] <= current_time:
